@@ -31,6 +31,9 @@ sub from_rgb { ($_[2] & 0xff) | (($_[1] & 0xff) << 8) | (($_[0] & 0xff) << 16) }
 sub to_rgb   { (( $_[0]>>16) & 0xFF, ($_[0]>>8) & 0xFF, $_[0] & 0xFF) }
 sub from_bgr { ($_[0] & 0xff) | (($_[1] & 0xff) << 8) | (($_[2] & 0xff) << 16) }
 sub to_bgr   { ($_[0] & 0xFF, ($_[0]>>8) & 0xFF, ( $_[0]>>16) & 0xFF) }
+sub to_gray_byte   { my ( $r, $g, $b ) = to_rgb($_[0]); return int(( $r + $g + $b ) / 3 + .5) }
+sub to_gray_rgb    { return from_rgb( (to_gray_byte($_[0])) x 3 ) }
+sub from_gray_byte { return 0x10101 * ($_[0] & 0xff) }
 sub premultiply { from_rgb( map { int($_ * $_[1] / 255) } to_rgb($_[0]) ) }
 
 package
@@ -71,6 +74,17 @@ package
     fp; *AUTOLOAD =  \&Prima::Const::AUTOLOAD;	# fill styles & font pitches
 package
     le; *AUTOLOAD =  \&Prima::Const::AUTOLOAD;	# line ends
+
+sub Arrow { [
+	conic => [1,0,2.5,0,2.5,-0.5],
+	line  => [0,2.5],
+	conic => [-2.5,-0.5,-2.5,0,-1,0]
+] }
+
+sub Rhombus { [
+	line  => [1,0,1.5,1,0,4,-1.5,1,-1,0],
+] }
+
 package
     lj; *AUTOLOAD =  \&Prima::Const::AUTOLOAD;	# line joins
 package
@@ -969,8 +983,8 @@ See L<Prima::Drawable/Raster operations>
 
         rop::Clear       # = 0
         rop::Xor         # = src ( 1 - dstA ) + dst ( 1 - srcA )
-        rop::SrcOver     # = src + dst (1 - srcA) 
-        rop::DstOver     # = dst + src (1 - dstA)
+        rop::SrcOver     # = src srcA + dst (1 - srcA)
+        rop::DstOver     # = dst srcA + src (1 - dstA)
         rop::SrcCopy     # = src
         rop::DstCopy     # = dst
         rop::SrcIn       # = src dstA
@@ -980,6 +994,9 @@ See L<Prima::Drawable/Raster operations>
         rop::SrcAtop     # = src dstA + dst ( 1 - srcA )
         rop::DstAtop     # = dst srcA + src ( 1 - dstA )
 
+        rop::Blend       # src + dst (1 - srcA)
+	                 # same as rop::SrcOver but assumes premultiplied source
+
         rop::PorterDuffMask - masks out all bits but the constants above
 
 Photoshop operators
@@ -1008,6 +1025,7 @@ Constant alpha flags
 Others
 
         rop::AlphaCopy
+	rop::ConstantColor
 
 =head2 sbmp:: - system bitmaps indices
 
@@ -1106,6 +1124,7 @@ See also L<Prima::Application/get_system_value>
 	sv::MenuCheckSize    - width and height of default menu check icon
 	sv::FriBidi          - 1 if Prima is compiled with libfribidi and full bidi unicode support is available
 	sv::Antialias        - 1 if system supports antialiasing and alpha layer for primitives
+	sv::LibThai          - 1 if Prima is compiled with libthai
 
 =head2 ta::  - alignment constants
 
