@@ -379,6 +379,8 @@ SIZE_EVENT:;
 		my-> set_centered( self, var-> growMode & gmXCenter, var-> growMode & gmYCenter);
 	if ( !event-> gen. B)
 		my-> first_that( self, (void*) Widget_size_notify, &event-> gen. R);
+	var-> w = event-> gen. P. x;
+	var-> h = event-> gen. P. y;
 	if ( doNotify) {
 		Point oldSize;
 		oldSize. x = event-> gen. R. left;
@@ -533,7 +535,14 @@ void Widget_handle_event( Handle self, PEvent event)
 		case cmMouseEnter:
 			my-> notify( self, "<siP", "MouseEnter", event-> pos. mod, event -> pos. where);
 			objCheck;
-			if ( prima_guts.application && is_opt( optShowHint) && (P_APPLICATION-> options. optShowHint) && var-> hint && SvCUR(var-> hint))
+			if (
+				prima_guts.application &&
+				is_opt( optShowHint) &&
+				(P_APPLICATION-> options. optShowHint) &&
+				var-> hint &&
+				SvTYPE(var->hint) != SVt_NULL &&
+				SvCUR(var-> hint)
+			)
 				C_APPLICATION-> set_hint_action( prima_guts.application, self, true, true);
 			break;
 		case cmMouseLeave:
