@@ -41,12 +41,9 @@
 
 #define COLORSET_3D                COLOR_LIGHT3D, COLOR_DARK3D
 
-#define COLORSET_GRAY              COLORSET_GRAY_NORMAL, COLORSET_GRAY_HILITE, \
-											COLORSET_GRAY_DISABLED, COLORSET_3D
-#define COLORSET_ALT_GRAY          COLORSET_GRAY_NORMAL, COLORSET_GRAY_ALT_HILITE, \
-											COLORSET_GRAY_DISABLED, COLORSET_3D
-#define COLORSET_PANEL             COLORSET_PANEL_NORMAL, COLORSET_PANEL_HILITE, \
-											COLORSET_PANEL_DISABLED, COLORSET_3D
+#define COLORSET_GRAY              COLORSET_GRAY_NORMAL,  COLORSET_GRAY_HILITE,     COLORSET_GRAY_DISABLED,  COLORSET_3D
+#define COLORSET_ALT_GRAY          COLORSET_GRAY_NORMAL,  COLORSET_GRAY_ALT_HILITE, COLORSET_GRAY_DISABLED,  COLORSET_3D
+#define COLORSET_PANEL             COLORSET_PANEL_NORMAL, COLORSET_PANEL_HILITE,    COLORSET_PANEL_DISABLED, COLORSET_3D
 
 static Color standard_button_colors[]      = { COLORSET_GRAY     };
 static Color standard_checkbox_colors[]    = { COLORSET_GRAY     };
@@ -85,11 +82,12 @@ static Color* standard_colors[] = {
 	standard_application_colors,
 };
 
-static const int MAX_COLOR_CLASS = sizeof( standard_colors) / sizeof( standard_colors[ 0]) - 1;
+static const int MAX_COLOR_CLASS = sizeof(standard_colors) / sizeof( standard_colors[ 0]) - 1;
 
 Color **
-prima_standard_colors(void)
+prima_standard_colors(int * n_classes)
 {
+	if ( n_classes ) *n_classes = MAX_COLOR_CLASS + 1;
 	return standard_colors;
 }
 
@@ -213,10 +211,7 @@ prima_allocate_color( Handle self, Color color, Brush * brush)
 	a[2] = COLOR_B(color);
 
 	if ( self && XF_LAYERED(XX) )
-		return brush->primary =
-			(((a[0] << guts. argb_bits. red_range  ) >> 8) << guts. argb_bits.   red_shift) |
-			(((a[1] << guts. argb_bits. green_range) >> 8) << guts. argb_bits. green_shift) |
-			(((a[2] << guts. argb_bits. blue_range ) >> 8) << guts. argb_bits.  blue_shift);
+		return brush->primary = DEV_RGB(&guts.argb_bits,a[0],a[1],a[2]);
 
 	if ( guts. grayScale) {
 		a[0] = a[1] = a[2] = ( a[0] + a[1] + a[2]) / 3;
@@ -384,10 +379,7 @@ ENOUGH:;
 					prima_color_add_ref( self, brush-> secondary, RANK_NORMAL);
 			}
 		} else
-			brush-> primary =
-				(((a[0] << guts. screen_bits. red_range  ) >> 8) << guts. screen_bits.   red_shift) |
-				(((a[1] << guts. screen_bits. green_range) >> 8) << guts. screen_bits. green_shift) |
-				(((a[2] << guts. screen_bits. blue_range ) >> 8) << guts. screen_bits.  blue_shift);
+			brush-> primary = DEV_RGB(&guts.screen_bits, a[0],a[1],a[2]);
 	}
 	return brush-> primary;
 }
