@@ -214,7 +214,7 @@ create_group( Handle self, Handle owner, Bool syncPaint, Bool clipOwner,
 			sys last_size. x = r. right  - r. left;
 			sys y_override = sys last_size. y = r. bottom - r. top;
 			sys handle = frame;
-			SetWindowLongPtr( frame, GWLP_USERDATA, ( LONG) self);
+			SetWindowLongPtr( frame, GWLP_USERDATA, (LONG_PTR) self);
 		}
 		break;
 	case WC_CUSTOM:
@@ -249,8 +249,7 @@ create_group( Handle self, Handle owner, Bool syncPaint, Bool clipOwner,
 	sys owner   = ownerView;
 	if ( !reset)
 		sys pointer = LoadCursor( guts. instance, IDC_ARROW);
-	SetWindowLongPtr( ret, GWLP_USERDATA, ( LONG) self);
-
+	SetWindowLongPtr( ret, GWLP_USERDATA, (LONG_PTR) self);
 	if ( reset)
 	{
 		int i;
@@ -1099,7 +1098,6 @@ apc_widget_begin_paint( Handle self, Bool insideOnPaint)
 			sys ps  = dc;
 			sys stock_bitmap = SelectObject( dc, bm);
 			sys bm = bm;
-			apc_gp_set_transform( self, -r. left, -r. top);
 			sys transform2. x = r. left;
 			sys transform2. y = r. top;
 			apt_set( aptBitmap);
@@ -1129,7 +1127,6 @@ apc_widget_begin_paint( Handle self, Bool insideOnPaint)
 		dsys( owner) ps = sys ps;
 		dsys(owner) transform2. x += ed. x;
 		dsys(owner) transform2. y += so. y - sz. y - ed. y;
-		apc_gp_set_transform( owner, 0, 0);
 		apc_gp_set_text_out_baseline( owner, dsys(owner) options. aptTextOutBaseline);
 
 		saved_dc_state = SaveDC( sys ps );
@@ -1154,17 +1151,12 @@ apc_widget_begin_paint( Handle self, Bool insideOnPaint)
 		RestoreDC( sys ps, saved_dc_state);
 
 		dsys(owner)transform2 = tr;
-		apc_gp_set_transform( owner, 0, 0);
-		dsys( owner) ps = dc;
+		dsys(owner)ps = dc;
 		CWidget( owner)-> end_paint( owner);
 	}
 
 	hwnd_enter_paint( self);
 
-	if ( useRPDraw) {
-		apc_gp_set_transform( self, sys transform. x, sys transform. y);
-	}
-
 	if ( is_apt(aptLayeredPaint)) {
 		Rect r;
 		r. left   = 0;
